"""
Testing the python3 SAX parser for loss of data when processing
a large file. 

Usage:     python3   EnchXMLTest.py   EnchTestData.html
"""

import sys
import string

# Use the SAX XML parsing library
import xml.sax

# ==========================================================================

# The ContentGenerator class is derived from the SAX handler
class ContentGenerator(xml.sax.handler.ContentHandler):

    # Establish initial state of parse actions
    # Redirect parser output to STDOUT stream at startup
    def __init__(self, std = sys.stdout):
        xml.sax.handler.ContentHandler.__init__(self)
        self.out = std
        self.iline  = 0
        self.ifield = 0
        self.field  = ''
        self.txtflg = False
        self.record = None
 
    # Event handlers for encountering new opening tags
    def startElement(self, name, attrs):
        # If it is a <tr> tag marking a new record, initialize record parse
        if name=='tr':
            self.ifield = -1
            self.iline = self.iline + 1
            self.record = [ ]

        # If it is a <td> tag marking each data field, increment the field 
        # count. 
        elif name=='td':
            self.ifield = self.ifield + 1

        # If it is a <p> tag delimiting field content, enable observation of text. 
        elif name=='p':
            self.txtflg = True

    # Event handlers for encountering closing tags
    def endElement(self, name):

        # If it is a </p> tag terminating text content, disable recording of text. 
        if name=='p':
            self.txtflg = False

        # If it is a </td> tag terminating field, append new field to record. 
        elif name=='td':
            # Append the field into the current data record
            self.record.append(self.field)
            if self.iline > 209 :
                token = (self.iline, self.ifield, self.field )
                self.out.write('\nline=%s   field=%s   content=%s' %  token)

        # Process a completed data record
        elif name=='tr':
            if self.iline > 209 :
                self.out.write('\nRecord %d processing completed.  Check field 2.' % self.iline)

        # Parsing is completed
        elif name=='body':
            # Send end-of-dataset flag
            self.out.write('\nEnd of file reached\n')


    # Receive notification that item body text has been captured.
    def characters(self, content):
        if  self.txtflg :
            self.field = content
 
# ==========================================================================

# Main program

# Construct a SAX parser
parser = xml.sax.make_parser()

# Configured the parser to use the ContentGenerator as the parser's event handler.
parser.setContentHandler(ContentGenerator())

# Run the SAX parser, specifying as its source the file from
# the command line argument.
parser.parse(sys.argv[1])

### end of program

